/* ----------------------------------------------------------------------------
 * Raima Database Manager
 *
 * Copyright (c) 2011 Raima Inc.,  All rights reserved.
 *
 * Use of this software, whether in source code format, or in executable,
 * binary object code form, is governed by the Birdstep LICENSE which
 * is fully described in the LICENSE.TXT file, included within this
 * distribution of files.
 * ----------------------------------------------------------------------------
 */

/** \file cpp-db.h
 *  \brief Private header for the C++ db API
 */

#if !defined(CPP_DB_H_INCLUDED_)
#define CPP_DB_H_INCLUDED_

#if !defined(__cplusplus)
#error cpp-db.h is only for C++ applications.  Do not include it in C applications
#endif

#include "rdmtypes.h"
#include "rdmrowidtypes.h"
#if !defined(RDM_NO_CPP_STD_STRING)
#include <string>
#endif

/** \brief The RDM C++ Namespace
 */
namespace RDM_CPP {

    class TFS;
    class Transaction;
    class IDb;
    class Cursor;
    typedef const void *dbType;
    class Db;
} // namespace RDM_CPP

/** \brief The Db base class

   This Db class provides the base database implementation.

   This is a class with very little overhead - a single pointer - that
   is a 'smart pointer' (i.e. will automatically get destroyed when the last
   reference is removed).

   To keep the overhead down this class should have no virtual functions.
*/
class RDM_PUBLIC_CLASS RDM_CPP::Db
{
  public:
    /* Handle more than one reference to the underlying database */

    /*! \brief Db Default Constructor

        The default constructor, copy constructor, assignment operator, and
        destructor implement a smart pointer for the Db interface and it's
        generated subclasses.  The number of references to the underlying
        implementation object will be maintained.  When the number of references
        to an implementation object is down to zero the object will be released.

        The default constructor will initialize the number of references to the
        underlying implementation object to one.
    */
    Db (void) throw ();

    /*! \brief Db Copy Constructor

        The default constructor, copy constructor, assignment operator, and
        destructor implement a smart pointer for the Db interface and it's
        generated subclasses.  The number of references to the underlying
        implementation object will be maintained.  When the number of references
        to an implementation object is down to zero the object will be released.

        The copy constructor will increment the number of references to the
        underlying implementation object by one.
    */
    Db (const Db &db) throw ();

    /*! \brief Db assignment operator

        The default constructor, copy constructor, assignment operator, and
        destructor implement a smart pointer for the Db interface and it's
        generated subclasses.  The number of references to the underlying
        implementation object will be maintained.  When the number of references
        to an implementation object is down to zero the object will be released.

        The assignment operator will increment the number of references to the
        underlying implementation object by one.
    */
    Db &operator= (const Db &db) throw ();

    /*! \brief Get the TFS object associated with the Db

        Each active Db is associated with a TFS object.  This method allows
        a user to get a reference to the TFS object associated with the source
        Db.

        \return The TFS object associated with this Db

        \par Locking Requirements
        None

        \throws
        rdm_exception
    */
    TFS GetTFS (void);

    /** \brief Removes all rows from all tables

       This method removes all rows from all database tables.

       This method must be the only change operations in an update transaction.
       An exception will be thrown if an update transaction is not active or if
       any other change operations have occurred in the update transaction prior
       to calling the DeleteAllRows() method.

       \par Locking Requirements
       An Update transaction with locks on all tables in the database

       \throws
       rdm_exception (::eINVDBID, ::eNOSTARTUPDATE, ::eINUPDATE)
    */
    void DeleteAllRows (void) const;

    /** \brief Removes all rows from a table

       This method removes all rows from the specified table

       This method must be the only change operations in an update transaction.
       An exception will be thrown if an update transaction is not active or if
       any other change operations have occurred in the update transaction prior
       to calling the DeleteAllRows() method.

       \par Locking Requirements
       An Update transaction with locks on the table (and all related tables)

       \throws
       rdm_exception (::eINVDBID, ::eNOSTARTUPDATE, ::eINUPDATE)
    */
    void DeleteAllRows (RDM_TABLE_ID tableId) const;

    /** \brief Set a database option

       This function is used to set any Tfs option. The \a optString is a key
       value pair in the form of \c{"key=value"}.

       \includedoc dox/sqldb-options.mdinc

    \par Locking Requirements
      None

    \throws
      rdm_exception (::eDBOPEN, ::eNOMEMORY)
    */
    void SetOptions (const char *optString) const;
#if !defined(RDM_NO_CPP_STD_STRING)
    void SetOptions (const std::string &optString) const;
#endif
    void SetOption (const char *option, const char *value) const;
#if !defined(RDM_NO_CPP_STD_STRING)
    void SetOption (const std::string &option, const std::string &value) const;
#endif

#if defined(RDM_UNICODE)
    /** \overload
     */
    void SetOptions (const wchar_t *optString) const;
#if !defined(RDM_NO_CPP_STD_STRING)
    void SetOptions (const std::wstring &optString) const;
#endif
    void SetOption (const wchar_t *option, const wchar_t *value) const;
#if !defined(RDM_NO_CPP_STD_STRING)
    void SetOption (const std::wstring &option, const std::wstring &value) const;
#endif
#endif

    /** \brief Begin reading from a database

       Use this method before reading data from the database.  Specify
       the tables you want access to or ::RDM_LOCK_ALL for every table.
       You are not allowed to combine this method with BeginSnapshot().
       Call End() when you are done reading.

       If the types parameter is NULL terminated there is no need to specify
       the number of items in the array in the num parameter.  However if the
       types parameter is not NULL terminated an exception can occur if the
       correct value for num is not provided.

       \par Locking Requirements
       None.

       \throws
       rdm_exception (::eINVDBID, ::eUNAVAIL, ::eINVIDENTIFIER)
    */
    Transaction StartRead (
        const RDM_TABLE_ID tableIds[] =
            NULL,        /**< [in] An array of table ids to be read locked */
        uint32_t num = 0 /**< [in] The number of tables to be locked */
        ) const;

    /** \brief Begin updating a database
        \param writeTableIds An array of table ids to be write locked
        \param numWriteTableIds The number of tables to be locked
        \param readTableIds An array of table ids to be read locked
        \param numReadTableIds The number of tables to be locked

       Use this method before making any updates to the database.
       Specify the tables you want access to or ::RDM_LOCK_ALL for every
       table.  You are not allowed to combine this method with
       BeginRead().  Call End() when you are finished with the update

       If the types parameter is NULL terminated there is no need to specify
       the number of items in the array in the num parameter.  However if the
       types parameter is not NULL terminated an exception can occur if the
       correct value for num is not provided.

       \par Locking Requirements
       None.

       \throws
       rdm_exception (::eINVDBID, ::eUNAVAIL, ::eINVIDENTIFIER, ::eREADONLY)
    */
    Transaction StartUpdate (
        const RDM_TABLE_ID writeTableIds[], /**< [in] An array of table
                                               identifiers to be read locked */
        uint32_t numWriteTableIds, /**< [in] The number of tables to be write-locked */
        const RDM_TABLE_ID readTableIds[] =
            NULL, /**< [in] An array of table identifiers to be read locked */
        uint32_t numReadTableIds =
            0 /**< [in] The number of tables to be read-locked */
        ) const;

    /** \overload
     */
    Transaction StartUpdate (
        const RDM_TABLE_ID writeTableIds[] =
            NULL, /**< [in] An array of table identifiers to be read locked */
        const RDM_TABLE_ID readTableIds[] =
            NULL, /**< [in] An array of table identifiers to be read locked */
        uint32_t numReadTableIds =
            0 /**< [in] The number of tables to be read-locked */
        ) const;

    /** \brief Create a static snapshot of the database

       Take a snapshot of the databases for reading data from the
       database.  You are not allowed to combine this method with
       BeginRead() or BeginSnapshot().  Call End() when you are done
       reading.

       \par Locking Requirements
       None.

       \throws
       rdm_exception (::eINVDBID)
    */
    Transaction StartSnapshot (
        const RDM_TABLE_ID tableIds[] =
            NULL, /**< [in] An array of table identifiers to be included in the snapshot */
        uint32_t num = 0 /**< [in] The number of tables in the snapshot */
        ) const;

    /** \brief Precommit a database transaction

       This methods prepares to commit a transaction by writing to the
       database log file all changes made since the most recent call to
       BeginUpdate(). Following soon after this call, a End() should be
       called to finally commit the transaction, or EndRollback() to throw it
       away.  The difference between calling Precommit prior to End() is that
       all file I/O necessary to prepare the transaction will be done during the
       Precommit, and the End() has a minimal task, essentially flipping a
       switch to commit the prepared transaction.

       This call does not change the status of locks, so it shouldn't be held
       in this state indefinitely.

       No updates or reads should be performed on the database after the
       Precommit until the end of the transaction.

       \par Locking Requirements
       None

       \throws
       rdm_exception (::eINVDBID)
    */
    void Precommit (void) const;

    /** \brief End a transaction

       This method ends an update, read or snapshot transaction.

       If the current transaction is an update all changes since the
       BeginUpdate() will be written to the database. The changes are
       first written to the log file to support recovery and are then written
       to the database by the TFS.

       End() frees all locked files for the database.

       \par Locking Requirements
       None

       \throws
       rdm_exception (::eINVDBID)
    */
    void End (void) const;

    /** \brief Rollback a transaction

       The method discards all database changes made between the call to
       StartUpdate() and the call to EndRollback().

       EndRollback() frees all locked tables for the database.

       \par Locking Requirements
       None

       \throws
       rdm_exception (::eINVDBID)
    */
    void EndRollback (void) const;

    /** \brief Get the current database transaction status

       \returns A ::RDM_TRANS_STATUS value

       \par Locking Requirements
       None

       \throws
       rdm_exception (::eINVDBID)
    */
    RDM_TRANS_STATUS GetTransactionStatus (void) const;

    /** \brief Check to see the database is opened exclusively

       \returns A boolean value
       \retval true the database is opened in exclusive mode
       \retval false the database is not opened in exclusive mode

       \par Locking Requirements
       None

       \throws
       rdm_exception (::eINVDBID)
    */
    bool IsOpenedExclusive (void) const;

    /** \brief Check to see the database is opened in shared mode

       \returns A boolean value
       \retval true the database is opened in shared mode
       \retval false the database is not opened in shared mode

       \par Locking Requirements
       None

       \throws
       rdm_exception (::eINVDBID)
    */
    bool IsOpenedShared (void) const;

    /** \brief Check to see the database is opened in read-only mode

       \returns A boolean value
       \retval true the database is opened in read-only mode
       \retval false the database is not opened in read-only mode

       \par Locking Requirements
       None

       \throws
       rdm_exception (::eINVDBID)
    */
    bool IsOpenedReadOnly (void) const;

    /** \brief Rebuild keys

       This method rebuilds a keys for the database.  Rebuilding keys
       is a two-step process. First, the index is reinitialized. Then, each
       row that has a key stored in the key file is sequentially read from
       the database and the key inserted into the index.

       This utility can be used to regenerate an index when dbcheck reports
       an inconsistency.

       \par Locking Requirements
       This method requires exclusive access to the database (opened in
       exclusive mode)

       \throws
       rdm_exception (::eINVDBID, ::eEXCLUSIVE, ::eNOMEMORY, ::eREADONLY)
    */
    void BuildKeys (
        const RDM_CHAR_T *optString /**< [in] The BuildKeys option string */
    );

#if defined(RDM_UNICODE)
    void BuildKeys (const wchar_t *optString);
#endif

    /** \brief Export database into text files

    This method exports data from the database to  ASCII text files. The
    method writes all data from each table to a separate file; it is the
    table name with file extension ".txt". If a table name is specified in
    the parameter structure then only that table's rows are
    exported. The created files will consist of one text line per row
    with sep_char (default ',') characters separating the columns.

    The Export method has two ways of printing extended ASCII characters to
    the output files. The default is to write out the octal value. Since the
    Import method cannot handle this, the extended option can be used to
    print the extended ASCII character as the character. For example, ASCII 2
    would be printed as "\002" without the extended and as a smiley face with
    the extended option.

    See dbexp for full details

    \par Locking Requirements
    A Snapshot transaction or a Read transaction with locks on all the database
    tables that will be exported

    \throws
    rdm_exception (::eINVDBID, ::eNOTLOCKED, ::eNOMEMORY)

    \lib{cppexport}
    */
    void Export (const char *optString /**< [in] Export options string */
                 ) const;
#if !defined(RDM_NO_CPP_STD_STRING)
    void Export (const std::string &optString /**< [in] Export options string */
                 ) const;
#endif

#if defined(RDM_UNICODE)
    void Export (const wchar_t *optString) const;
#if !defined(RDM_NO_CPP_STD_STRING)
    void Export (const std::wstring &optString) const;
#endif
#endif

    /** \brief Import data from text files

        This method imports data into the database from ASCII text files.  The
        import functionality is simple enough to allow great quantities of raw
        ASCII data to be entered into an RDM database with very little
        specification. With more complex requirements, ImportXML can be
       instructed to perform mappings between the input data and RDM tables and
       to make set connections based upon matching column values.

        All instructions to Import are placed into a text file in the form of an
        import specification language (ISL). An ISL specification identifies the
        database and the files containing ASCII data to be imported. It also
        contains mappings between the ASCII data fields and the columns.

        \par Locking Requirements
        An Update transaction with locks on all tables that will be imported

        \throws
        rdm_exception (::eINVDBID, ::eNOSTARTUPDATE, ::eNOTLOCKED, ::sDUPLICATE)

        \lib{cppexport}
    */
    void Import (const char *optString /**< [in] import options string */
                 ) const;
#if !defined(RDM_NO_CPP_STD_STRING)
    void Import (const std::string &optString /**< [in] import options string */
                 ) const;
#endif
#if defined(RDM_UNICODE)
    void Import (const wchar_t *optString) const;
#if !defined(RDM_NO_CPP_STD_STRING)
    void Import (const std::wstring &optString /**< [in] import options string */
                 ) const;
#endif
#endif

    //! Get the internal RDM_DB
    /*!
    \return The internal RDM_DB
    */
    RDM_DB GetDb ();

    //! Get current return code value.

    /*!

    The Db object does not throw exceptions for status codes.  This function
    allows you to get the return code from the last method call.  If the
   previous method call did not throw and exception this will either be sOKAY or
   a status code value.  If the previous method call did throw an exception this
   will be the error code related to that exception.

   \return The current RDM_RETCODE value
    */
    RDM_RETCODE GetReturnCode () const;

    /*! \internal

        \brief Release the database

        The default constructor, copy constructor, assignment operator, and
        destructor implement a smart pointer for the Db interface and it's
        generated subclasses.  The number of references to the underlying
        implementation object will be maintained.  When the number of references
        to an implementation object is down to zero the object will be released.

        The constructor will initialize the number of references to the
        underlying implementation object to one.
    */
    Db &Release (void);

    /*! \brief Db destructor

        The default constructor, copy constructor, assignment operator, and
        destructor implement a smart pointer for the Db interface and it's
        generated subclasses.  The number of references to the underlying
        implementation object will be maintained.  When the number of references
        to an implementation object is down to zero the object will be released.

        The destructor will decrement the number of references to the underlying
        implementation object by one.
    */
    ~Db ();

    /** \brief Associate a catalog with an allocated database

    This method associates a catalog with a Db.  The Db must not have a database
    open.  When the database is opened with \ref Open method the associated catalog
    will be used.  If a catalog is found on the TFS then the specified catalog and
    the stored catalog will be compared to ensure that they are identical.
*/
    Db SetCatalog (const RDM_CHAR_T *const catalog = NULL) const;
#if defined(RDM_UNICODE)
    Db SetCatalog (const wchar_t *const catalog) const;
#endif

    /** \brief Compile a catalog and associate it with an allocated database

    This method compiles a catalog and associates it with a Db.  The
    Db must not have a database open.  When the database is opened
    with \ref Open method the associated catalog will be used.  If a
    catalog is found on the TFS then the specified catalog and the
    stored catalog will be compared to ensure that they are identical.
*/
    Db CompileCatalog (const RDM_CHAR_T *const catalog = NULL) const;
#if defined(RDM_UNICODE)
    Db CompileCatalog (const wchar_t *const catalog) const;
#endif

    /** \brief Load a catalog and associate it with an allocated database

    This method loads a catalog and associates it with a Db.  The
    Db must not have a database open.  When the database is opened
    with \ref Open method the associated catalog will be used.  If a
    catalog is found on the TFS then the specified catalog and the
    stored catalog will be compared to ensure that they are identical.
*/
    Db LoadCatalog (const RDM_CHAR_T *const catalog = NULL) const;
#if defined(RDM_UNICODE)
    Db LoadCatalog (const wchar_t *const catalog) const;
#endif

    /** \brief Associate a catalog stored in a file with an allocated database

    This method associates a catalog with a Db.  The Db must not have a database
    open.  When the database is opened with Open() method the associated catalog
    will be used.  If a catalog is found on the TFS then the specified catalog
    and the stored catalog will be compared to ensure that they are identical.
*/
    Db SetCatalogFromFile (const RDM_CHAR_T *const file) const;
#if defined(RDM_UNICODE)
    Db SetCatalogFromFile (const wchar_t *const file) const;
#endif

    /** \brief Compile a catalog stored on file and associate it with an allocated database

    This method compiles a catalog stored on file and associates it
    with a Db.  The Db must not have a database open.  When the
    database is opened with Open() method the associated catalog will
    be used.  If a catalog is found on the TFS then the specified
    catalog and the stored catalog will be compared to ensure that
    they are identical.
*/
    Db CompileCatalogFromFile (const RDM_CHAR_T *const file) const;
#if defined(RDM_UNICODE)
    Db CompileCatalogFromFile (const wchar_t *const file) const;
#endif

    /** \brief Load a catalog stored on file and associate it with an allocated database

    This method loads a catalog stored on file and associates it
    with a Db.  The Db must not have a database open.  When the
    database is opened with Open() method the associated catalog will
    be used.  If a catalog is found on the TFS then the specified
    catalog and the stored catalog will be compared to ensure that
    they are identical.
*/
    Db LoadCatalogFromFile (const RDM_CHAR_T *const file) const;
#if defined(RDM_UNICODE)
    Db LoadCatalogFromFile (const wchar_t *const file) const;
#endif

    /** \brief Alter a catalog with an allocated database

    This method alters a catalog with a Db.
*/
    Db AlterCatalog (const RDM_CHAR_T *const catalog) const;
#if defined(RDM_UNICODE)
    Db AlterCatalog (const wchar_t *const catalog) const;
#endif

    /** \brief Get the current lock status of a table

    This method returns the current lock status of the specified table.
    If the Db does not have a lock on the table then the status will be ::RDM_LOCK_FREE
    regardless of whether any other sessions have the table locked.  This function
    allows the user to know if the Db currently has access to the table.
    */

    RDM_LOCK_STATUS GetLockStatus (RDM_TABLE_ID tableId) const;

    /** \brief Add an encryption context to the Db

    This method will associate an encryption context to a Db.
    When a database is opened the system will use the encryption contexts
    associated with the Db to attempt to open the database.

    Typically you would associate the encryption context to a handle prior to
    opening a database.

    You can remove all encryption contexts associated with the Db by passing
    in a NULL encryption context to this method.

    */
    void SetEncrypt (RDM_ENCRYPT enc) const;

    /** \brief Get the encryption context associated with the Db

    This method allows the application to retrive the encryption context
    associated with a Db object. It is possible that multiple encryption
    contexts are associated with a Db object. If that is the case this
    function will return the most recently associated encryption context.

    */
    RDM_ENCRYPT GetEncrypt (void) const;

    /** \brief Encrypt a database

    This method can encrypt, decrypt, and re-encrypt a Db

    When re-encrypting a database you can choose whether to do a full or
    quick re-encryption. A quick re-encryption will just change the passcode
    used for encrypting the randomly generated key used to encrypt the actual
    database contents. In this case the database contents do not need to be
    read and rewritten so the operation can be performed very quickly.

    A full re-encryption will generate a new internal encryption key and will
    require each database page to be read, decrypted, re-encrypted, and
    rewritten to the file. This can be a lengthy operation for very large
    database images.

    To decrypt an encrypted database pass in a NULL encryption context (enc).

    \par Locking Requirements
    This function requires that the database be opened in exclusive mode

    \throws
    rdm_exception
    */
    void Encrypt (RDM_ENCRYPT enc, const RDM_CHAR_T *optString) const;
#if defined(RDM_UNICODE)
    void Encrypt (RDM_ENCRYPT enc, const wchar_t *optString) const;
#endif

    /** \brief Vacuum a database

    This method can vacuum a database.

    When vacuuming a database you move database objects into the current pack
    file.  When earlier pack files no longer contain any database objects
    they can be deleted.  This is the only method available to reclaim disk
    space.

    \par Locking Requirements
    None

    \throws
    rdm_exception
    */
    void Vacuum (const RDM_CHAR_T *optString) const;
#if defined(RDM_UNICODE)
    void Vacuum (const wchar_t *optString) const;
#endif

    /** \brief Flush the id index

    This method will flush the database's id index

    \par Locking Requirements
    None

    \throws
    rdm_exception
    */
    void FlushIdIndex () const;

    /** \brief Rebuild the id index

    This method will rebuild the database's id index

    \par Locking Requirements
    None

    \throws
    rdm_exception
    */
    void RebuildIdIndex () const;

    /** \brief Generate a replication buffer

    This method generates a replication buffer of recent changes that can be
    applied to another database via a call to the ReplicationReceive method

    \par Locking Requirements
    Read lock on all tables

    \throws
    rdm_exception
    */
    void ReplicationSend (
        void *replicationBuffer,
        size_t bufferSize,
        size_t &bytesWritten) const;

    /** \brief Process a replication buffer

    This method processes a replication buffer of recent changes and applies
    them to the database.

    \par Locking Requirements
    Database must be opened in read-only mode

    \throws
    rdm_exception
    */
    void ReplicationReceive (const void *replicationBuffer, size_t bufferSize) const;


    /** \brief Create a new pack file

    This method will create a new pack file.

    \par Locking Requirements
    None

    \throws
    rdm_exception
    */
    void CreateNewPackFile () const;

    /** \brief Drop an open database
     */
    void Drop ();

    /** \brief Open the database
     */
    Db Open (
        const RDM_CHAR_T *name,
        RDM_OPEN_MODE mode,
        const RDM_CHAR_T *const catalog) const;
    /** \overload
     */
    Db Open (const RDM_CHAR_T *name, RDM_OPEN_MODE mode = RDM_OPEN_SHARED) const;

#if defined(RDM_UNICODE)
    Db Open (const wchar_t *name, RDM_OPEN_MODE mode = RDM_OPEN_SHARED) const;
    Db Open (const wchar_t *name, RDM_OPEN_MODE mode, const char *const catalog) const;
    Db Open (const char *name, RDM_OPEN_MODE mode, const wchar_t *const catalog) const;
    Db Open (const wchar_t *name, RDM_OPEN_MODE mode, const wchar_t *const catalog)
        const;
#endif

    /** \brief Close the database
     */
    Db Close () const;

    /** \brief Clear the runtime cache
     */
    void ClearCache (void) const;

    /** \brief Turn on the processing of triggers
     */
    void TriggersOn (void) const;

    /** \brief Turn off the processing of triggers
     */
    void TriggersOff (void) const;

    /** \brief Check the triggers status
     */
    RDM_TRIGGERS_STATUS TriggersStatus (void) const;

    /** \brief Rebuild an index
     */
    void RebuildKey (
        RDM_KEY_ID keyId,
        uint32_t reportFrequency = 0,
        RDM_REBUILD_INDEX_REPORT_FCN *fcn = NULL) const;

    /** \brief Rebuild multiples indexes
     */
    void RebuildKeys (
        uint32_t numKeys,
        const RDM_KEY_ID *keyIds,
        uint32_t reportFrequency = 0,
        RDM_REBUILD_INDEX_REPORT_FCN *fcn = NULL) const;

    /** \brief Persist an in-memory database
     */
    void Persist (void) const;

/** \brief Get a single database option into a string

    This function retrieves a single database option into a string value. The
    \a key parameter accepts the key for the option to get.

    \includedoc dox/sqldb-options.mdinc

*/
#if !defined(RDM_NO_CPP_STD_STRING)
    std::string GetOption (std::string option) const;
#endif
    /** \overload
     */
    void GetOption (
        const char *option,
        char *value,
        size_t bytesIn,
        size_t *bytesOut = NULL) const;

#if defined(RDM_UNICODE)
    /** \overload
     */
#if !defined(RDM_NO_CPP_STD_STRING)
    std::wstring GetOption (std::wstring option) const;
#endif
    /** \overload
     */
    void GetOption (
        const wchar_t *option,
        wchar_t *value,
        size_t bytesIn,
        size_t *bytesOut = NULL) const;
#endif

/** \brief Get database information into a string

    This function retrieves database information into a string value. The
    \a key parameter accepts the key for the information to get.

*/
#if !defined(RDM_NO_CPP_STD_STRING)
    std::string GetInfo (std::string key) const;
#endif
    /** \overload
     */
    void GetInfo (
        const char *key,
        char *value,
        size_t bytesIn,
        size_t *bytesOut = NULL) const;

#if defined(RDM_UNICODE)
    /** \overload
     */
#if !defined(RDM_NO_CPP_STD_STRING)
    std::wstring GetInfo (std::wstring key) const;
#endif
    /** \overload
     */
    void GetInfo (
        const wchar_t *key,
        wchar_t *value,
        size_t bytesIn,
        size_t *bytesOut = NULL) const;
#endif

    /** \brief Get database memory usage information

    This method retrieves database memory usage information

    The memory usage is split between system memory which is memory that is used
    by the engine and user data which is memory used for storing the user data
    (rows, keys, blobs, schema, etc.).

    For a disk based database the user memory usage will always be 0 as the data
    is stored on disk and not in-memory.

    Both disk and in-memory databases require system memory.

    */
    void GetMemoryUsage (
        uint64_t &systemCurr, /**< [out] The current amount of system memory in
                                 use for the database (does not include user data - rows, keys, etc.) */
        uint64_t &systemMax, /**< [out] The maximum total amount of system memory
                                in use for the database (does not include user data - rows, keys, etc.) */
        uint64_t &userCurr, /**< [out] The current amount of memory for user data in the database */
        uint64_t &userMax /**< [out] The maximum amount of memory for user data in the database */
        ) const;

#ifndef RDM_DOXYGEN
  protected:
    IDb *m_idb;
    Db (IDb *idb);
    Db (const Db &, dbType);
    Cursor _InsertRow (
        RDM_TABLE_ID,
        const void *,
        size_t = 0,
        RDM_ROWID_T rowid = NULL_ROWID) const;
    Cursor _InsertRows (RDM_TABLE_ID, const void *, uint32_t, size_t, Cursor &) const;
    Cursor _InsertRows (RDM_TABLE_ID, const void *, uint32_t, Cursor &) const;
    Cursor _InsertRows (RDM_TABLE_ID, const void *, uint32_t) const;
    Cursor _InsertRows (RDM_TABLE_ID, const void *, uint32_t, size_t) const;

    Cursor _GetRows (RDM_TABLE_ID, RDM_ROWID_T = NULL_ROWID) const;
    Cursor _GetRowsByKey (RDM_KEY_ID id) const;
    Cursor _GetRowsByRtreeKey (RDM_KEY_ID id) const;
    Cursor _GetRowsByKey (RDM_KEY_ID, const void *, size_t = 0) const;
    Cursor _GetRowsByKey (RDM_KEY_ID, const RDM_SEARCH_KEY *) const;
    Cursor _GetRowsByKey (RDM_KEY_ID, const void *, size_t, RDM_RTREE_TYPE, uint32_t)
        const;
    Cursor _GetRowsByKeyInRange (
        RDM_KEY_ID,
        const RDM_SEARCH_KEY *,
        const RDM_SEARCH_KEY * = NULL) const;
    Cursor _GetRowsByKeyInRange (
        RDM_KEY_ID,
        const RDM_RANGE_KEY *,
        const RDM_RANGE_KEY * = NULL) const;
    Cursor _GetRowsByKeyInRange (
        RDM_KEY_ID,
        const void *,
        size_t = 0,
        const void * = NULL,
        size_t = 0) const;
    dbType _GetType (void) const;

    void _VerifyType (dbType) const;
    void _ValidateIDb () const;
    Db &_AssignAndVerifyType (dbType, const Db &);

  private:
    int16_t __countLocks (const RDM_TABLE_ID[], int16_t) const;

    //! Db reference operator
    const Db *operator& (void) const;
#endif

    friend class Cursor;
    friend class TFS;
};
#endif /* CPP_DB_H_INCLUDED_ */
