/*
 * Raima Database Manager
 *
 * Copyright (C) 2016 Raims Inc.,  All rights reserved.
 *
 * Use of this software, whether in source code format, or in executable,
 * binary object code form, is governed by the Raima LICENSE which
 * is fully described in the LICENSE.TXT file, included within this
 * distribution of files.
 */

/** \file psptypes.h
 *  \brief Header for the Platform Support Package - Platform dependent types
 */
#if !defined(PSPTYPES_H_INCLUDED_)
#define PSPTYPES_H_INCLUDED_

#include "pspplat.h"

#if defined(_lint)
#include "psptypes-lint.h"
#endif

#if defined(RDM_DOXYGEN)
#include "psptypes-doxygen.h"
#else
#if defined(RDM_WINDOWS)
#include "psptypes-win.h"
#elif defined(RDM_VXWORKS)
#include "psptypes-vxworks.h"
#elif defined(RDM_INTEGRITY)
#include "psptypes-integrity.h"
#elif defined(RDM_UNIX)
#include "psptypes-unix.h"
#elif defined(RDM_FREE_RTOS)
#include "psptypes-free_rtos.h"
#elif defined(RDM_NO_OS)
#include "psptypes-noos.h"
#endif

#include <stddef.h>
#endif

#ifndef RDM_UNREF
#define RDM_UNREF(a) ((void) a)
#endif

#ifndef RDM_UNSET
#define RDM_UNSET(a) = a
#endif

/** Enumeration for RDM boolen values
 *
 *   This enumeration adds support for a platform/language agnostic
 *   BOOLEAN value in the RDM library
 */
typedef enum
{
    RDM_FALSE = 0, /**< enum value for FALSE */
    RDM_TRUE = 1,   /**< enum value for TRUE  */
    __RDM_BOOL_MAX = 0x7fffffff
} RDM_BOOL_T;

/** Enumeration for RDM comparison values
 */
typedef enum
{
    RDM_LT = -1, /**< The first argument is lesser than the second argument */
    RDM_EQ = 0,  /**< The arguments are equal */
    RDM_GT = 1   /**< The first argument is greater than the second argument */
} RDM_COMPARE;

typedef struct RDM_ENCRYPT_S *RDM_ENCRYPT; /**< RDM encryption handle */

/** Enumeration for RDM encryption algorithms
 *
 *   The enumeration identifies the supported encryption algorithms.
 *
 *   RDM_ENC_XOR is the only type of encryption that is included in
 *   packages without export restrictions.
 */
typedef enum
{
    RDM_ENC_NONE,   /**< No encryption */
    RDM_ENC_XOR,    /**< XOR encryption */
    RDM_ENC_AES128, /**< AES-128 encryption */
    RDM_ENC_AES192, /**< AES-192 encryption */
    RDM_ENC_AES256, /**< AES-256 encryption */
    RDM_ENC_DEFAULT /**< Default encryption */
} RDM_ENC_TYPE;

#define EXTERNAL_FCN_PTR EXTERNAL_FCN *

#if !defined(RDM_STARTUP_HOOK)
/** Startup hook
 *
 *  Startup hook for tools, examples, and tests that are using any of
 *  the RDM_STARTUP macros */
#define RDM_STARTUP_HOOK
#endif
#if !defined(RDM_SHUTDOWN_HOOK)
/** Shutdown hook
 *
 *  Shutdown hook for tools, examples, and tests that are using any of
 *  the RDM_STARTUP macros */
#define RDM_SHUTDOWN_HOOK
#endif

#if defined(RDM_DOXYGEN)
/** RDM character
 *
 *  RDM_CHAR_T are always expanded to char. On Windows extra
 *  overloaded methods are provided with parameter types wchar_t
 *  instead of char */
#define RDM_CHAR_T RDM_CHAR_T
#else
#define RDM_CHAR_T char
#endif

#if defined(_UNICODE) && defined(RDM_UNICODE)
#define RDM_T(text) L##text
#define RDM_TCHAR_T wchar_t
#define RDM_TSTR_FMT "%ls"
#else
/** Macro for an RDM string */
#define RDM_T(text) text
#if defined(RDM_DOXYGEN)
/** RDM_TCHAR_T is expanded to wchar_t on Windows when _UNICODE is
 * defined.  Otherwise it is expanded to char. */
#define RDM_TCHAR_T RDM_TCHAR_T
#else
#define RDM_TCHAR_T char
#endif
#define RDM_TSTR_FMT "%s"
#endif

#if !defined(RDM_UNICODE)
#define RDM_T_FUNC(func) func
#else
#if defined(_UNICODE)
#define RDM_T_FUNC(func) func##W
#else
#define RDM_T_FUNC(func) func##A
#endif
#endif

#if defined(__cplusplus)
extern "C" {
#endif
RDM_EXPORT int32_t psp_threadMain (
    int32_t (*main) (int32_t argc, const char *const *argv),
    int argc,
    const RDM_TCHAR_T *const *argv
    );
#if defined(__cplusplus)
}
#endif

#if !defined(RDM_STARTUP_EXAMPLE)
/** Startup for examples
 *
 * This sets up a main function, with a startup hook, a call to a
 * named main function, and a shutdown hook.
 *
 * This is used for examples.
 */
#define RDM_STARTUP_EXAMPLE(name)                                              \
    int32_t name##_main (int32_t argc, const RDM_TCHAR_T *const *argv);        \
    int main (int argc, const RDM_TCHAR_T *const *argv)                        \
    {                                                                          \
        int32_t status;                                                        \
                                                                               \
        RDM_STARTUP_HOOK                                                       \
        status = psp_threadMain (name##_main, argc, argv);                     \
        RDM_SHUTDOWN_HOOK                                                      \
        return status;                                                         \
    }
#endif

#if !defined(RDM_DOXYGEN)
#if !defined(RDM_STARTUP_TEST)
#define RDM_STARTUP_TEST(name)                                                 \
    int32_t name##_main (int32_t argc, const char *const *argv);               \
    int main (int argc, const char *const *argv)                               \
    {                                                                          \
        int32_t status;                                                        \
                                                                               \
        RDM_STARTUP_HOOK                                                       \
        status = psp_threadMain (name##_main, argc, argv);                     \
        RDM_SHUTDOWN_HOOK                                                      \
        return status;                                                         \
    }
#endif
#endif

/** Macro to get the number of elements in an array of any type
 */
#define RDM_LEN(x) (sizeof (x) / sizeof (x[0]))

#if defined(NDEBUG) || defined(RDM_NO_ASSERT)
#define RDM_ASSERT(x)
#elif defined(RDM_ASSERT_INFINITE_LOOP)
#define RDM_ASSERT(x)                                                          \
    {                                                                          \
        if (!(x))                                                              \
        {                                                                      \
            int asserted = 1;                                                  \
            while (asserted)                                                   \
                ;                                                              \
        }                                                                      \
    }
#define RDM_ASSERT_ENABLED
#else
#include <assert.h>
/** Macro for doing assertions.  Will be turned off if NDEBUG is defined */
#define RDM_ASSERT(x) assert (x)
#define RDM_ASSERT_ENABLED
#endif

#if defined(RDM_ASSERT_ENABLED)
#define RDM_EVAL_ASSERT(x, y) RDM_ASSERT (x == y)
#else
/** Macro for doing assertions on x == y.  Will always evaluate x, even when
 * NDEBUG (or not RDM_ASSERT_ENABLED) is defined */
#define RDM_EVAL_ASSERT(x, y) x
#endif

/** Macro fo taking the min of two arguments of any type */
#define RDM_MIN(a, b) ((a) < (b) ? (a) : (b))
/** Macro fo taking the max of two arguments of any type */
#define RDM_MAX(a, b) ((a) > (b) ? (a) : (b))

/** Minimum value for int_t */
#define RDM_INT8_MIN ((int8_t) 0x80U)
/** Maximum value for int64_t */
#define RDM_INT8_MAX ((int8_t) 0x7FU)
/** Maximum value for uint64_t */
#define RDM_UINT8_MAX ((uint8_t) 0xFFU)
/** Minimum value for int16_t */
#define RDM_INT16_MIN ((int16_t) 0x8000U)
/** Maximum value for int64_t */
#define RDM_INT16_MAX ((int16_t) 0x7FFFU)
/** Maximum value for uint64_t */
#define RDM_UINT16_MAX ((uint16_t) 0xFFFFU)
/** Minimum value for int32_t */
#define RDM_INT32_MIN ((int32_t) 0x80000000U)
/** Maximum value for int64_t */
#define RDM_INT32_MAX ((int32_t) 0x7FFFFFFFU)
/** Maximum value for uint64_t */
#define RDM_UINT32_MAX ((uint32_t) 0xFFFFFFFFU)
/** Minimum value for int64_t */
#define RDM_INT64_MIN (-9223372036854775807LL - 1)
/** Maximum value for int64_t */
#define RDM_INT64_MAX (9223372036854775807LL)
/** Maximum value for uint64_t */
#define RDM_UINT64_MAX (18446744073709551615ULL)

/** Format string for 64 bit integer */
#define RDM_INT64_FMT RDM_T ("%") RDM_64BIT_CODE RDM_T ("d")
/** Format string for 64 bit unsigned integer */
#define RDM_UINT64_FMT RDM_T ("%") RDM_64BIT_CODE RDM_T ("u")
/** Format string for 64 bit integer in hexadecimal */
#define RDM_INT64_FMT_HEX RDM_T ("%") RDM_64BIT_CODE RDM_T ("X")
/** Format string for 64 bit unsigned integer in hexadecimal */
#define RDM_UINT64_FMT_HEX RDM_T ("%") RDM_64BIT_CODE RDM_T ("X")
/** Format string for 64 bit integer where size is specified */
#define RDM_INT64_FMT_SIZE(size) RDM_T ("%") #size RDM_64BIT_CODE RDM_T ("d")
/** Format string for 64 bit unsigned integer where size is specified */
#define RDM_UINT64_FMT_SIZE(size) RDM_T ("%") #size RDM_64BIT_CODE RDM_T ("u")
/** Format string for 64 bit integer in hexadecimal where size is specified */
#define RDM_INT64_FMT_SIZE_HEX(size)                                           \
    RDM_T ("%") #size RDM_64BIT_CODE RDM_T ("X")
/** Format string for 64 bit unsigned integer in hexadecimal where size is specified */
#define RDM_UINT64_FMT_SIZE_HEX(size)                                          \
    RDM_T ("%") #size RDM_64BIT_CODE RDM_T ("X")

#if !defined(RDM_DOXYGEN)
#define RDM_INT64_FMT_A "%" RDM_64BIT_CODE_A "d"
#define RDM_UINT64_FMT_A "%" RDM_64BIT_CODE_A "u"
#define RDM_INT64_FMT_HEX_A "%" RDM_64BIT_CODE_A "X"
#define RDM_UINT64_FMT_HEX_A "%" RDM_64BIT_CODE_A "X"
#define RDM_INT64_FMT_SIZE_A(size) "%" #size RDM_64BIT_CODE_A "d"
#define RDM_UINT64_FMT_SIZE_A(size) "%" #size RDM_64BIT_CODE_A "u"
#define RDM_INT64_FMT_SIZE_HEX_A(size) "%" #size RDM_64BIT_CODE_A "X"
#define RDM_UINT64_FMT_SIZE_HEX_A(size) "%" #size RDM_64BIT_CODE_A "X"
#endif

/* Two levels of macros are needed to stringify the result of expansion
 * of a macro argumen */
#define RDM_STRINGIFY_(data) #data
#define RDM_STRINGIFY(data) RDM_STRINGIFY_ (data)

#if !defined(RDM_BYTES_PER_UTF8_CHAR)
#define RDM_BYTES_PER_UTF8_CHAR 4
#else
#if RDM_BYTES_PER_UTF8_CHAR < 1 || RDM_BYTES_PER_UTF8_CHAR > 4
#error "RDM_BYTES_PER_UTF8_CHAR must be between 1 and 4 (inclusive)"
#endif
#endif

#define RDM_BYTES_PER_UTF8_CHAR_OPTION                                         \
    ("bytes_per_utf8_char=" RDM_STRINGIFY (RDM_BYTES_PER_UTF8_CHAR))
#define RDM_BYTES_PER_UTF8_CHAR_OPTION_VALUE                                   \
    RDM_STRINGIFY (RDM_BYTES_PER_UTF8_CHAR)

#endif /* PSPTYPES_H_INCLUDED_ */
