/*
 * Raima Database Manager
 *
 * Copyright (C) 2016 Raims Inc.,  All rights reserved.
 *
 * Use of this software, whether in source code format, or in executable,
 * binary object code form, is governed by the Raima LICENSE which
 * is fully described in the LICENSE.TXT file, included within this
 * distribution of files.
 */

/** \file rdmbcdtypes.h
 *  \brief Header for Binary-coded decimal (BCD) types
 */

#if !defined(RDMBCDTYPES_H_INCLUDED_)
#define RDMBCDTYPES_H_INCLUDED_

#include "psptypes.h"

/** \brief The length of a RDM BCD in bytes given by its precission */
#define RDM_BCDLEN(prec) ((uint16_t) (offsetof (RDM_BCD_T, data) + (prec) + 1))

/** \brief Rounding modes for RDM BCD */
typedef enum
{
    RDM_BCD_ROUND_NEAREST_TIES_TO_EVEN, /**< This is the default (because it is
                                           the recommended default for decimal
                                           numbers in IEEE 754). When rounding
                                           is necessary, the digits that are to
                                           be discarded are investigated and
                                           rounding takes place towards the nearer
                                           value. If these digits put the value
                                           exactly in the middle (i.e., the first
                                           digit to be truncated is 5 and all
                                           other truncated digits are 0) then
                                           the digit before the truncated digits
                                           is investigated and if it is even the
                                           data is rounded towards it - otherwise
                                           it is rounded away from it. The idea
                                           is to minimize rounding bias by rounding
                                           one way half of the time and rounding
                                           the other way the other half.*/
    RDM_BCD_ROUND_NEAREST_TIES_AWAY_FROM_ZERO, /**< This is what most of us are
                                                  familiar with for rounding.Similar
                                                  to the previous one, but 'ties'
                                                  (when the truncated digits are
                                                  50000...) are rounded away from zero.*/
    RDM_BCD_ROUND_UP,   /**< All values are rounded up; meaning positive values
                           are rounded away from zero and negative values are   rounded   towards zero.*/
    RDM_BCD_ROUND_DOWN, /**< All values are rounded down; meaning positive
                           values are rounded towards zero and negative values
                           are rounded away from zero.*/
    RDM_BCD_ROUND_TOWARDS_ZERO /**< All values are rounded towards zero.*/
} RDM_BCD_ROUNDING_MODE;

/** \brief The max presission for a RDM BCD */
#define RDM_BCD_MAXPRECISION 32
/* \brief The max scale for a RDM BCD */
#define RDM_BCD_MAXSCALE 32

/** \brief The RDM Binary Coded Decimal (BCD) data structure
 */
typedef struct
{
    uint8_t prec; /**< The number of decimals after the desimal point */
    int8_t exp; /**< The exponent in base 10 that the fraction should be adjusted with */
    uint8_t flags; /**< For internal use only.  Should be set to 0 */
    int8_t sign; /**< The sign were 1 indicate a positive numbers and -1 a negative numbers */
    uint8_t data[RDM_BCD_MAXPRECISION + 1]; /**< Fraction */
} RDM_BCD_T;

#endif /* RDMBCDTYPES_H_INCLUDED_ */
